# PAY.JP Python bindings

# Configuration variables

api_key = None
api_base = 'https://api.pay.jp'
api_version = None

# TODO include Card?
__all__ = ['Account', 'Card', 'Charge', 'Customer', 'Event', 'Plan', 'Subscription', 'Token', 'Transfer']

# Resource
from payjp.resource import (  # noqa
    Account, Charge, Customer, Event, Plan, Subscription, Token, Transfer)




# coding: utf-8

import textwrap
import warnings

from payjp import error

import requests


def new_default_http_client(*args, **kwargs):
    impl = RequestsClient

    return impl(*args, **kwargs)


class HTTPClient(object):

    def request(self, method, url, headers, post_data=None):
        raise NotImplementedError(
            'HTTPClient subclasses must implement `request`')


class RequestsClient(HTTPClient):
    name = 'requests'

    def request(self, method, url, headers, post_data=None):
        kwargs = {}

        try:
            try:
                result = requests.request(method,
                                          url,
                                          headers=headers,
                                          data=post_data,
                                          timeout=80,
                                          **kwargs)
            except TypeError as e:
                raise TypeError(
                    'Warning: It looks like your installed version of the '
                    '"requests" library is not compatible with Payjp\'s '
                    'usage thereof. (HINT: The most likely cause is that '
                    'your "requests" library is out of date. You can fix '
                    'that by running "pip install -U requests".) The '
                    'underlying error was: %s' % (e,))

            # This causes the content to actually be read, which could cause
            # e.g. a socket timeout. TODO: The other fetch methods probably
            # are susceptible to the same and should be updated.
            content = result.content
            status_code = result.status_code
        except Exception as e:
            # Would catch just requests.exceptions.RequestException, but can
            # also raise ValueError, RuntimeError, etc.
            self._handle_request_error(e)
        return content, status_code

    def _handle_request_error(self, e):
        if isinstance(e, requests.exceptions.RequestException):
            msg = ("Unexpected error communicating with Payjp.  "
                   "If this problem persists, let us know at "
                   "support@payjp.com.")
            err = "%s: %s" % (type(e).__name__, str(e))
        else:
            msg = ("Unexpected error communicating with Payjp. "
                   "It looks like there's probably a configuration "
                   "issue locally.  If this problem persists, let us "
                   "know at support@payjp.com.")
            err = "A %s was raised" % (type(e).__name__,)
            if str(e):
                err += " with error message %s" % (str(e),)
            else:
                err += " with no error message"
        msg = textwrap.fill(msg) + "\n\n(Network error: %s)" % (err,)
        raise error.APIConnectionError(msg)



# coding: utf-8

import payjp

payjp.api_key = 'sk_test_c62fade9d045b54cd76d7036'

print('Attempting charge...')

resp = payjp.Charge.create(
    amount=10,
    currency='jpy',
    card={
        'number': '4242424242424242',
        'exp_month': 12,
        'exp_year': 2018
    },
    description='a TIROL Choco'
)

print(resp)
print(('Success: %r') % (resp, ))




# coding: utf-8

import datetime
import json
import logging
import sys

from six import string_types
from six.moves.urllib.parse import quote_plus

from . import (
    api_requestor,
    error,
    util,
)

logger = logging.getLogger('payjp')

def convert_to_payjp_object(resp, api_key, account, api_base=None):
    types = {'account': Account, 'card': Card,
             'charge': Charge, 'customer': Customer,
             'event': Event, 'plan': Plan,
             'subscription': Subscription, 'token': Token,
             'transfer': Transfer, 'list': ListObject}

    if isinstance(resp, list):
        return [convert_to_payjp_object(i, api_key, account, api_base) for i in resp]
    elif isinstance(resp, dict) and not isinstance(resp, PayjpObject):
        resp = resp.copy()
        klass_name = resp.get('object')
        if isinstance(klass_name, string_types):
            klass = types.get(klass_name, PayjpObject)
        else:
            klass = PayjpObject
        return klass.construct_from(resp, api_key, payjp_account=account, api_base=api_base)
    else:
        return resp

def _compute_diff(current, previous):
    if isinstance(current, dict):
        previous = previous or {}
        diff = current.copy()
        for key in set(previous.keys()) - set(diff.keys()):
            diff[key] = ""
        return diff
    return current if current is not None else ""

def _serialize_list(array, previous):
    array = array or []
    previous = previous or []
    params = {}

    for i, v in enumerate(array):
        previous_item = previous[i] if len(previous) > i else None
        if hasattr(v, 'serialize'):
            params[str(i)] = v.serialize(previous_item)
        else:
            params[str(i)] = _compute_diff(v, previous_item)

    return params


class PayjpObject(dict):

    def __init__(self, id=None, api_key=None, payjp_account=None, api_base=None, **kwargs):
        super(PayjpObject, self).__init__()

        self._unsaved_values = set()
        self._transient_values = set()

        self._retrieve_params = kwargs
        self._previous = None
        self._api_base = api_base

        object.__setattr__(self, 'api_key', api_key)
        object.__setattr__(self, 'payjp_account', payjp_account)

        if id:
            self['id'] = id

    def __setattr__(self, k, v):
        if k[0] == '_' or k in self.__dict__:
            return super(PayjpObject, self).__setattr__(k, v)
        else:
            self[k] = v

    def __getattr__(self, k):
        if k[0] == '_':
            raise AttributeError(k)

        try:
            return self[k]
        except KeyError as err:
            raise AttributeError(*err.args)

    def __setitem__(self, k, v):
        if v == "":
            raise ValueError(
                "You cannot set %s to an empty string. "
                "We interpret empty strings as None in requests."
                "You may set %s.%s = None to delete the property" % (
                    k, str(self), k))

        super(PayjpObject, self).__setitem__(k, v)

        # Allows for unpickling in Python 3.x
        if not hasattr(self, '_unsaved_values'):
            self._unsaved_values = set()

        self._unsaved_values.add(k)

    def __getitem__(self, k):
        try:
            return super(PayjpObject, self).__getitem__(k)
        except KeyError as err:
            if k in self._transient_values:
                raise KeyError(
                    "%r.  HINT: The %r attribute was set in the past."
                    "It was then wiped when refreshing the object with "
                    "the result returned by PAY.JP's API, probably as a "
                    "result of a save().  The attributes currently "
                    "available on this object are: %s" %
                    (k, k, ', '.join(self.keys())))
            else:
                raise err

    def __delitem__(self, k):
        raise TypeError(
            "You cannot delete attributes on a PayjpObject. "
            "To unset a property, set it to None.")

    @classmethod
    def construct_from(cls, values, key, payjp_account=None, api_base=None):
        instance = cls(values.get('id'), api_key=key,
                       payjp_account=payjp_account)
        instance.refresh_from(values, api_key=key,
                              payjp_account=payjp_account,
                              api_base=api_base)
        return instance

    def refresh_from(self, values, api_key=None, partial=False,
                     payjp_account=None, api_base=None):
        self.api_key = api_key or getattr(values, 'api_key', None)
        self.payjp_account = \
            payjp_account or getattr(values, 'payjp_account', None)
        if self.api_base is not None:
            self._api_base = api_base

        # Wipe old state before setting new.  This is useful for e.g.
        # updating a customer, where there is no persistent card
        # parameter.  Mark those values which don't persist as transient
        if partial:
            self._unsaved_values = (self._unsaved_values - set(values))
        else:
            removed = set(self.keys()) - set(values)
            self._transient_values = self._transient_values | removed
            self._unsaved_values = set()
            self.clear()

        self._transient_values = self._transient_values - set(values)

        for k, v in values.items():
            super(PayjpObject, self).__setitem__(
                k, convert_to_payjp_object(v, api_key, payjp_account, api_base))

        self._previous = values

    def serialize(self, previous):
        params = {}
        unsaved_keys = self._unsaved_values or set()
        previous = previous or self._previous or {}

        for k, v in self.items():
            if k == 'id' or (isinstance(k, str) and k.startswith('_')):
                continue
            elif isinstance(v, APIResource):
                continue
            elif hasattr(v, 'serialize'):
                params[k] = v.serialize(previous.get(k, None))
            elif k in unsaved_keys:
                params[k] = _compute_diff(v, previous.get(k, None))
            elif k == 'additional_owners' and v is not None:
                params[k] = _serialize_list(v, previous.get(k, None))

        return params

    def api_base(self):
        return self._api_base

    def request(self, method, url, params=None, headers=None):
        if params is None:
            params = self._retrieve_params
        requestor = api_requestor.APIRequestor(
            key=self.api_key, api_base=self.api_base(),
            account=self.payjp_account)
        response, api_key = requestor.request(method, url, params, headers)

        return convert_to_payjp_object(response, api_key, self.payjp_account, self.api_base())

    def __repr__(self):
        ident_parts = [type(self).__name__]

        if isinstance(self.get('object'), string_types):
            ident_parts.append(self.get('object'))

        if isinstance(self.get('id'), string_types):
            ident_parts.append('id=%s' % (self.get('id'),))

        unicode_repr = '<%s at %s> JSON: %s' % (
            ' '.join(ident_parts), hex(id(self)), str(self))

        if sys.version_info[0] < 3:
            return unicode_repr.encode('utf-8')
        else:
            return unicode_repr

    def __str__(self):
        return json.dumps(self, sort_keys=True, indent=2)


class ListObject(PayjpObject):

    def all(self, **params):
        return self.request('get', self['url'], params)

    def create(self, **params):
        # TODO divide into another parent class
        if hasattr(self, 'object') and self.object == 'list' and \
                hasattr(self, 'count') and self.count > 0 and \
                isinstance(self.data[0], Subscription):
                    raise NotImplementedError(
                        "Can't create a subscription via customer object. "
                        "Use payjp.Subscription.create({'customer_id'}) instead.")
        return self.request('post', self['url'], params)

    def retrieve(self, id, **params):
        base = self.get('url')
        id = util.utf8(id)
        extn = quote_plus(id)
        url = "%s/%s" % (base, extn)

        return self.request('get', url, params)


class APIResource(PayjpObject):

    @classmethod
    def class_name(cls):
        return str(quote_plus(cls.__name__.lower()))

    @classmethod
    def class_url(cls):
        cls_name = cls.class_name()
        return '/v1/{0}s'.format(cls_name)

    def instance_url(self):
        id = self.get('id')
        if not id:
            raise error.InvalidRequestError("Could not create instance url without it's id",
                                            None)

        base = self.class_url()
        ext = quote_plus(id)
        return '{0}/{1}'.format(base, ext)

    @classmethod
    def retrieve(cls, id, api_key=None, payjp_account=None, api_base=None, **kwargs):
        instance = cls(id, api_key, payjp_account, api_base, **kwargs)
        instance.refresh()
        return instance

    def refresh(self):
        self.refresh_from(self.request('get', self.instance_url()))
        return self


class ListableAPIResource(APIResource):

    @classmethod
    def all(cls, api_key=None, payjp_account=None, api_base=None, **params):
        requestor = api_requestor.APIRequestor(api_key, account=payjp_account, api_base=api_base)
        url = cls.class_url()
        response, api_key = requestor.request('get', url, params)
        return convert_to_payjp_object(response, api_key, payjp_account, api_base)


class CreateableAPIResource(APIResource):

    @classmethod
    def create(cls, api_key=None, payjp_account=None, **params):
        requestor = api_requestor.APIRequestor(api_key, account=payjp_account)
        url = cls.class_url()
        response, api_key = requestor.request('post', url, params)
        return convert_to_payjp_object(response, api_key, payjp_account)


class UpdateableAPIResource(APIResource):

    def save(self):
        updated_params = self.serialize(None)

        if updated_params:
            self.refresh_from(self.request('post', self.instance_url(),
                                           updated_params))
        else:
            logger.debug("Trying to save already saved object %r", self)
        return self


class DeletableAPIResource(APIResource):

    def delete(self, **params):
        self.refresh_from(self.request('delete', self.instance_url(), params))
        return self

# resources

class Token(CreateableAPIResource):
    pass


class Charge(CreateableAPIResource, ListableAPIResource,
             UpdateableAPIResource):

    def capture(self, **kwargs):
        url = self.instance_url() + '/capture'
        self.refresh_from(self.request('post', url, kwargs))
        return self

    def refund(self, **kwargs):
        url = self.instance_url() + '/refund'
        self.refresh_from(self.request('post', url, kwargs))
        return self


class Event(ListableAPIResource):
    pass


class Customer(CreateableAPIResource, UpdateableAPIResource,
               ListableAPIResource, DeletableAPIResource):

    def charges(self, **params):
        params['customer'] = self.id
        charges = Charge.all(self.api_key, params)
        return charges


class Plan(CreateableAPIResource, DeletableAPIResource,
           UpdateableAPIResource, ListableAPIResource):
    pass


class Account(APIResource):

    @classmethod
    def retrieve(cls, id=None, api_key=None, payjp_account=None, api_base=None, **params):
        instance = cls(id, api_key, payjp_account, api_base, **params)
        instance.refresh()
        return instance

    def instance_url(self):
        id = self.get('id')
        if not id:
            return "/v1/accounts"
        id = util.utf8(id)
        base = self.class_url()
        extn = quote_plus(id)
        return "%s/%s" % (base, extn)


class Card(UpdateableAPIResource, DeletableAPIResource):

    def instance_url(self):
        self.id = util.utf8(self.id)
        extn = quote_plus(self.id)
        if (hasattr(self, 'customer')):
            self.customer = util.utf8(self.customer)

            base = Customer.class_url()
            owner_extn = quote_plus(self.customer)

        else:
            raise error.InvalidRequestError(
                "Could not determine whether card_id %s is "
                "attached to a customer "
                "or a recipient." % self.id, 'id')

        return "%s/%s/cards/%s" % (base, owner_extn, extn)

    @classmethod
    def retrieve(cls, id, api_key=None, payjp_account=None, api_base=None, **params):
        raise NotImplementedError(
            "Can't retrieve a card without a customer ID."
            "Use customer.cards.retrieve('card_id') or "
            "recipient.cards.retrieve('card_id') instead.")


class Subscription(CreateableAPIResource, DeletableAPIResource,
                   UpdateableAPIResource, ListableAPIResource):

    def pause(self, **kwargs):
        url = self.instance_url() + '/pause'
        self.refresh_from(self.request('post', url, kwargs))
        return self

    def resume(self, **kwargs):
        url = self.instance_url() + '/resume'
        self.refresh_from(self.request('post', url, kwargs))
        return self

    def cancel(self, **kwargs):
        url = self.instance_url() + '/cancel'
        self.refresh_from(self.request('post', url, kwargs))
        return self


class Transfer(ListableAPIResource):
    pass




# coding: utf-8

import json
import pickle
import sys
import time
import datetime
import unittest

import payjp
import payjp.resource

from payjp.test.helper import (
    PayjpUnitTestCase, PayjpApiTestCase,
    MyListable, MyCreatable, MyUpdateable, MyDeletable,
    MyResource, NOW, DUMMY_CARD, DUMMY_CHARGE,
    DUMMY_PLAN)


class PayjpObjectTests(PayjpUnitTestCase):

    def test_initializes_with_parameters(self):
        obj = payjp.resource.PayjpObject(
            'foo', 'bar', myparam=5, yourparam='boo')

        self.assertEqual('foo', obj.id)
        self.assertEqual('bar', obj.api_key)

    def test_access(self):
        obj = payjp.resource.PayjpObject('myid', 'mykey', myparam=5)

        # Empty
        self.assertRaises(AttributeError, getattr, obj, 'myattr')
        self.assertRaises(KeyError, obj.__getitem__, 'myattr')
        self.assertEqual('def', obj.get('myattr', 'def'))
        self.assertEqual(None, obj.get('myattr'))

        # Setters
        obj.myattr = 'myval'
        obj['myitem'] = 'itval'
        self.assertEqual('sdef', obj.setdefault('mydef', 'sdef'))

        # Getters
        self.assertEqual('myval', obj.setdefault('myattr', 'sdef'))
        self.assertEqual('myval', obj.myattr)
        self.assertEqual('myval', obj['myattr'])
        self.assertEqual('myval', obj.get('myattr'))

        self.assertEqual(['id', 'myattr', 'mydef', 'myitem'],
                         sorted(obj.keys()))
        self.assertEqual(['itval', 'myid', 'myval', 'sdef'],
                         sorted(obj.values()))

        # Illegal operations
        self.assertRaises(ValueError, setattr, obj, 'foo', '')
        self.assertRaises(TypeError, obj.__delitem__, 'myattr')

    def test_refresh_from(self):
        obj = payjp.resource.PayjpObject.construct_from({
            'foo': 'bar',
            'trans': 'me',
        }, 'mykey')

        self.assertEqual('mykey', obj.api_key)
        self.assertEqual('bar', obj.foo)
        self.assertEqual('me', obj['trans'])
        self.assertEqual(None, obj.payjp_account)

        obj.refresh_from({
            'foo': 'baz',
            'johnny': 5,
        }, 'key2', payjp_account='acct_foo')

        self.assertEqual(5, obj.johnny)
        self.assertEqual('baz', obj.foo)
        self.assertRaises(AttributeError, getattr, obj, 'trans')
        self.assertEqual('key2', obj.api_key)
        self.assertEqual('acct_foo', obj.payjp_account)

        obj.refresh_from({
            'trans': 4,
            'metadata': {'amount': 42}
        }, 'key2', True)

        self.assertEqual('baz', obj.foo)
        self.assertEqual(4, obj.trans)

    def test_passing_nested_refresh(self):
        obj = payjp.resource.PayjpObject.construct_from({
            'foos': {
                'type': 'list',
                'data': [
                    {'id': 'nested'}
                ],
            }
        }, 'key', payjp_account='acct_foo')

        nested = obj.foos.data[0]

        self.assertEqual('key', obj.api_key)
        self.assertEqual('nested', nested.id)
        self.assertEqual('key', nested.api_key)
        self.assertEqual('acct_foo', nested.payjp_account)

    def check_invoice_data(self, data):
        # Check rough structure
        self.assertEqual(20, len(data.keys()))
        self.assertEqual(3, len(data['lines'].keys()))
        self.assertEqual(0, len(data['lines']['invoiceitems']))
        self.assertEqual(1, len(data['lines']['subscriptions']))

        # Check various data types
        self.assertEqual(1338238728, data['date'])
        self.assertEqual(None, data['next_payment_attempt'])
        self.assertEqual(False, data['livemode'])
        self.assertEqual('month',
                         data['lines']['subscriptions'][0]['plan']['interval'])

    def test_repr(self):
        obj = payjp.resource.PayjpObject(
            'foo', 'bar', myparam=5)

        obj['object'] = u'\u4e00boo\u1f00'

        res = repr(obj)

        if sys.version_info[0] < 3:
            res = unicode(repr(obj), 'utf-8')

        self.assertTrue(u'<PayjpObject \u4e00boo\u1f00' in res)
        self.assertTrue(u'id=foo' in res)

    def test_pickling(self):
        obj = payjp.resource.PayjpObject(
            'foo', 'bar', myparam=5)

        obj['object'] = 'boo'
        obj.refresh_from({'fala': 'lalala'}, api_key='bar', partial=True)

        self.assertEqual('lalala', obj.fala)

        pickled = pickle.dumps(obj)
        newobj = pickle.loads(pickled)

        self.assertEqual('foo', newobj.id)
        self.assertEqual('bar', newobj.api_key)
        self.assertEqual('boo', newobj['object'])
        self.assertEqual('lalala', newobj.fala)


class ListObjectTests(PayjpApiTestCase):

    def setUp(self):
        super(ListObjectTests, self).setUp()

        self.lo = payjp.resource.ListObject.construct_from({
            'id': 'me',
            'url': '/my/path',
        }, 'mykey')

        self.mock_response([{
            'object': 'charge',
            'foo': 'bar',
        }])

    def assertResponse(self, res):
        self.assertTrue(isinstance(res[0], payjp.Charge))
        self.assertEqual('bar', res[0].foo)

    def test_all(self):
        res = self.lo.all(myparam='you')

        self.requestor_mock.request.assert_called_with(
            'get', '/my/path', {'myparam': 'you'}, None)

        self.assertResponse(res)

    def test_create(self):
        res = self.lo.create(myparam='eter')

        self.requestor_mock.request.assert_called_with(
            'post', '/my/path', {'myparam': 'eter'}, None)

        self.assertResponse(res)

    def test_retrieve(self):
        res = self.lo.retrieve('myid', myparam='cow')

        self.requestor_mock.request.assert_called_with(
            'get', '/my/path/myid', {'myparam': 'cow'}, None)

        self.assertResponse(res)


class APIResourceTests(PayjpApiTestCase):

    def test_instance_url_without_id(self):
        from payjp.error import InvalidRequestError
        resource = MyResource.construct_from({'id': None}, 'mykey')
        with self.assertRaises(InvalidRequestError):
            resource.instance_url()

    def test_retrieve_and_refresh(self):
        self.mock_response({
            'id': 'foo2',
            'bobble': 'scrobble',
        })

        res = MyResource.retrieve('foo*', myparam=5)

        url = '/v1/myresources/foo%2A'
        self.requestor_mock.request.assert_called_with(
            'get', url, {'myparam': 5}, None
        )

        self.assertEqual('scrobble', res.bobble)
        self.assertEqual('foo2', res.id)
        self.assertEqual('reskey', res.api_key)

        self.mock_response({
            'frobble': 5,
        })

        res = res.refresh()

        url = '/v1/myresources/foo2'
        self.requestor_mock.request.assert_called_with(
            'get', url, {'myparam': 5}, None
        )

        self.assertEqual(5, res.frobble)
        self.assertRaises(KeyError, res.__getitem__, 'bobble')

    def test_convert_to_payjp_object(self):
        sample = {
            'foo': 'bar',
            'adict': {
                'object': 'charge',
                'id': 42,
                'amount': 7,
            },
            'alist': [
                {
                    'object': 'customer',
                    'name': 'chilango'
                }
            ]
        }

        converted = payjp.resource.convert_to_payjp_object(
            sample, 'akey', None)

        # Types
        self.assertTrue(isinstance(converted, payjp.resource.PayjpObject))
        self.assertTrue(isinstance(converted.adict, payjp.Charge))
        self.assertEqual(1, len(converted.alist))
        self.assertTrue(isinstance(converted.alist[0], payjp.Customer))

        # Values
        self.assertEqual('bar', converted.foo)
        self.assertEqual(42, converted.adict.id)
        self.assertEqual('chilango', converted.alist[0].name)

        # Stripping
        # TODO: We should probably be stripping out this property
        # self.assertRaises(AttributeError, getattr, converted.adict, 'object')


class ListableAPIResourceTests(PayjpApiTestCase):

    def test_all(self):
        self.mock_response([
            {
                'object': 'charge',
                'name': 'jose',
            },
            {
                'object': 'charge',
                'name': 'curly',
            }
        ])

        res = MyListable.all()

        self.requestor_mock.request.assert_called_with(
            'get', '/v1/mylistables', {})

        self.assertEqual(2, len(res))
        self.assertTrue(all(isinstance(obj, payjp.Charge) for obj in res))
        self.assertEqual('jose', res[0].name)
        self.assertEqual('curly', res[1].name)


class CreateableAPIResourceTests(PayjpApiTestCase):

    def test_create(self):
        self.mock_response({
            'object': 'charge',
            'foo': 'bar',
        })

        res = MyCreatable.create()

        self.requestor_mock.request.assert_called_with(
            'post', '/v1/mycreatables', {})

        self.assertTrue(isinstance(res, payjp.Charge))
        self.assertEqual('bar', res.foo)

    def test_idempotent_create(self):
        self.mock_response({
            'object': 'charge',
            'foo': 'bar',
        })

        res = MyCreatable.create()

        self.requestor_mock.request.assert_called_with(
            'post', '/v1/mycreatables', {})

        self.assertTrue(isinstance(res, payjp.Charge))
        self.assertEqual('bar', res.foo)


class UpdateableAPIResourceTests(PayjpApiTestCase):

    def setUp(self):
        super(UpdateableAPIResourceTests, self).setUp()

        self.mock_response({
            'thats': 'it'
        })

        self.obj = MyUpdateable.construct_from({
            'id': 'myid',
            'foo': 'bar',
            'baz': 'boz',
            'metadata': {
                'size': 'l',
                'score': 4,
                'height': 10
            }
        }, 'mykey')

    def checkSave(self):
        self.assertTrue(self.obj is self.obj.save())

        self.assertEqual('it', self.obj.thats)
        # TODO: Should we force id to be retained?
        # self.assertEqual('myid', obj.id)
        self.assertRaises(AttributeError, getattr, self.obj, 'baz')

    def test_idempotent_save(self):
        self.obj.baz = 'updated'
        self.obj.save()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/myupdateables/myid',
            {
                'metadata': {},
                'baz': 'updated',
            },
            None
        )

    def test_save(self):
        self.obj.baz = 'updated'
        self.obj.other = 'newval'
        self.obj.metadata.size = 'm'
        self.obj.metadata.info = 'a2'
        self.obj.metadata.height = None

        self.checkSave()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/myupdateables/myid',
            {
                'baz': 'updated',
                'other': 'newval',
                'metadata': {
                    'size': 'm',
                    'info': 'a2',
                    'height': '',
                }
            },
            None
        )

    def test_add_key_to_nested_object(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'size': 'l',
                'score': 4,
                'height': 10
            }
        }, 'mykey')

        acct.legal_entity['first_name'] = 'bob'

        self.assertTrue(acct is acct.save())

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/myupdateables/myid',
            {
                'legal_entity': {
                    'first_name': 'bob',
                }
            },
            None
        )

    def test_save_nothing(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'metadata': {
                'key': 'value',
            }
        }, 'mykey')

        self.assertTrue(acct is acct.save())
        self.assertTrue(self.requestor_mock.request.called)

    def test_replace_nested_object(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'last_name': 'smith',
            }
        }, 'mykey')

        acct.legal_entity = {
            'first_name': 'bob',
        }

        self.assertTrue(acct is acct.save())

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/myupdateables/myid',
            {
                'legal_entity': {
                    'first_name': 'bob',
                    'last_name': '',
                }
            },
            None
        )

    def test_array_setting(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {}
        }, 'mykey')

        acct.legal_entity.additional_owners = [{'first_name': 'Bob'}]

        self.assertTrue(acct is acct.save())

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/myupdateables/myid',
            {
                'legal_entity': {
                    'additional_owners': [
                        {'first_name': 'Bob'}
                    ]
                }
            },
            None
        )

    def test_array_none(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'additional_owners': None,
            }
        }, 'mykey')

        acct.foo = 'bar'

        self.assertTrue(acct is acct.save())

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/myupdateables/myid',
            {
                'foo': 'bar',
                'legal_entity': {},
            },
            None
        )

    def test_array_insertion(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'additional_owners': []
            }
        }, 'mykey')

        acct.legal_entity.additional_owners.append({'first_name': 'Bob'})

        self.assertTrue(acct is acct.save())

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/myupdateables/myid',
            {
                'legal_entity': {
                    'additional_owners': {
                        '0': {'first_name': 'Bob'},
                    }
                }
            },
            None
        )

    def test_array_update(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'additional_owners': [
                    {'first_name': 'Bob'},
                    {'first_name': 'Jane'}
                ]
            }
        }, 'mykey')

        acct.legal_entity.additional_owners[1].first_name = 'Janet'

        self.assertTrue(acct is acct.save())

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/myupdateables/myid',
            {
                'legal_entity': {
                    'additional_owners': {
                        '0': {},
                        '1': {'first_name': 'Janet'}
                    }
                }
            },
            None
        )

    def test_array_noop(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'additional_owners': [{'first_name': 'Bob'}]
            },
            'currencies_supported': ['jpy', 'cad']
        }, 'mykey')

        self.assertTrue(acct is acct.save())

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/myupdateables/myid',
            {
                'legal_entity': {'additional_owners': {'0': {}}}
            },
            None
        )

    def test_hash_noop(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'address': {'line1': '1 Two Three'}
            }
        }, 'mykey')

        self.assertTrue(acct is acct.save())

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/myupdateables/myid',
            {'legal_entity': {'address': {}}},
            None
        )

    def test_save_replace_metadata_with_number(self):
        self.obj.baz = 'updated'
        self.obj.other = 'newval'
        self.obj.metadata = 3

        self.checkSave()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/myupdateables/myid',
            {
                'baz': 'updated',
                'other': 'newval',
                'metadata': 3,
            },
            None
        )

    def test_save_overwrite_metadata(self):
        self.obj.metadata = {}
        self.checkSave()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/myupdateables/myid',
            {
                'metadata': {
                    'size': '',
                    'score': '',
                    'height': '',
                }
            },
            None
        )

    def test_save_replace_metadata(self):
        self.obj.baz = 'updated'
        self.obj.other = 'newval'
        self.obj.metadata = {
            'size': 'm',
            'info': 'a2',
            'score': 4,
        }

        self.checkSave()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/myupdateables/myid',
            {
                'baz': 'updated',
                'other': 'newval',
                'metadata': {
                    'size': 'm',
                    'info': 'a2',
                    'height': '',
                    'score': 4,
                }
            },
            None
        )


class DeletableAPIResourceTests(PayjpApiTestCase):

    def test_delete(self):
        self.mock_response({
            'id': 'mid',
            'deleted': True,
        })

        obj = MyDeletable.construct_from({
            'id': 'mid'
        }, 'mykey')

        self.assertTrue(obj is obj.delete())

        self.assertEqual(True, obj.deleted)
        self.assertEqual('mid', obj.id)


class PayjpResourceTest(PayjpApiTestCase):

    def setUp(self):
        super(PayjpResourceTest, self).setUp()
        self.mock_response({})


class ChargeTest(PayjpResourceTest):

    def test_charge_list_all(self):
        payjp.Charge.all(created={'lt': NOW})

        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/charges',
            {
                'created': {'lt': NOW},
            }
        )

    def test_charge_list_create(self):
        payjp.Charge.create(**DUMMY_CHARGE)

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/charges',
            DUMMY_CHARGE,
        )

    def test_charge_list_retrieve(self):
        payjp.Charge.retrieve('ch_test_id')

        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/charges/ch_test_id',
            {},
            None
        )

    def test_charge_list_retrieve_with_api_key_and_api_base(self):
        payjp.Charge.retrieve('ch_test_id', api_key='KEY', api_base='BASE')
        self.requestor_class_mock.assert_called_with(key='KEY', api_base='BASE', account=None)
        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/charges/ch_test_id',
            {},
            None
        )

    def test_create_with_source_param(self):
        payjp.Charge.create(amount=100, currency='jpy',
                             source='btcrcv_test_receiver')

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/charges',
            {
                'amount': 100,
                'currency': 'jpy',
                'source': 'btcrcv_test_receiver'
            },
        )


class AccountTest(PayjpResourceTest):

    def test_retrieve_account(self):
        payjp.Account.retrieve('acct_foo')
        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/accounts/acct_foo',
            {},
            None
        )

    def test_retrieve_account_with_api_key_and_api_base(self):
        payjp.Account.retrieve('acct_foo', api_key='KEY', api_base='BASE')
        self.requestor_class_mock.assert_called_with(key='KEY', api_base='BASE', account=None)
        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/accounts/acct_foo',
            {},
            None
        )


class CustomerTest(PayjpResourceTest):

    def test_list_customers(self):
        payjp.Customer.all()
        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/customers',
            {},
        )

    def test_list_customers_with_api_key_and_api_base(self):
        payjp.Customer.all(api_key='KEY', api_base='BASE')
        self.requestor_class_mock.assert_called_with('KEY', account=None, api_base='BASE')
        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/customers',
            {},
        )

    def test_create_customer(self):
        payjp.Customer.create(description="foo bar", card=DUMMY_CARD)
        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers',
            {
                'description': 'foo bar',
                'card': DUMMY_CARD
            },
        )

    def test_unset_description(self):
        customer = payjp.Customer(id="cus_unset_desc")
        customer.description = "Hey"
        customer.save()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers/cus_unset_desc',
            {
                'description': 'Hey',
            },
            None
        )

    def test_cannot_set_empty_string(self):
        customer = payjp.Customer()
        self.assertRaises(ValueError, setattr, customer, "description", "")

    def test_customer_add_card(self):
        customer = payjp.Customer.construct_from({
            'id': 'cus_add_card',
            'cards': {
                'object': 'list',
                'url': '/v1/customers/cus_add_card/cards',
            },
        }, 'api_key')
        customer.cards.create(card=DUMMY_CARD)

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers/cus_add_card/cards',
            {
                'card': DUMMY_CARD,
            },
            None
        )


class TransferTest(PayjpResourceTest):

    def test_list_transfers(self):
        payjp.Transfer.all()
        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/transfers',
            {}
        )


class CustomerSubscriptionTest(PayjpResourceTest):

    def test_create_customer(self):
        payjp.Customer.create(plan=DUMMY_PLAN['id'], card=DUMMY_CARD)

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers',
            {
                'card': DUMMY_CARD,
                'plan': DUMMY_PLAN['id'],
            },
        )
    
    
    def test_list_customer_subscription(self):
        customer = payjp.Customer.construct_from({
            'id': 'cus_foo',
            'subscriptions': {
                'object': 'list',
                'url': '/v1/customers/cus_foo/subscriptions',
            }
        }, 'api_key')
        
        customer.subscriptions.all()
        
        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/customers/cus_foo/subscriptions',
            {},
            None
        )
    
    def test_retrieve_customer_subscription(self):
        customer = payjp.Customer.construct_from({
            'id': 'cus_foo',
            'subscriptions': {
                'object': 'list',
                'url': '/v1/customers/cus_foo/subscriptions',
            }
        }, 'api_key')

        customer.subscriptions.retrieve('sub_cus')

        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/customers/cus_foo/subscriptions/sub_cus',
            {},
            None
        )


class SubscriptionTest(PayjpResourceTest):

    def test_create_subscription(self):
        customer = payjp.Customer.construct_from({'id': 'cus_foo',}, 'api_key')
        payjp.Subscription.create(customer=customer.id, plan=DUMMY_PLAN['id'])

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/subscriptions',
            {'customer': customer.id,
            'plan': DUMMY_PLAN['id']},
        )

    def test_delete_subscriptions(self):
        sub = payjp.Subscription.construct_from(
            {'id': "sub_delete",
            'customer': "cus_foo",}
            , 'api_key')
        sub.delete()

        self.requestor_mock.request.assert_called_with(
            'delete',
            '/v1/subscriptions/sub_delete',
            {},
            None
        )

    def test_update_subscription(self):
        sub = payjp.Subscription.construct_from(
            {'id': "sub_update",
            'customer': "cus_foo",
            'plan': 'plan_foo'}
            , 'api_key')
        sub.plan = DUMMY_PLAN['id']
        sub.save()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/subscriptions/sub_update',
            {
                'plan': DUMMY_PLAN['id'],
            },
            None
        )
    
    def test_pause_subscriptions(self):
        sub = payjp.Subscription.construct_from(
            {'id': "sub_delete",
            'customer': "cus_foo",
            'status': 'active'}
            , 'api_key')
        sub.pause()
        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/subscriptions/sub_delete/pause',
            {},
            None
        )
    
    def test_cancel_subscriptions(self):
        sub = payjp.Subscription.construct_from(
            {'id': "sub_delete",
            'customer': "cus_foo",
            'status': 'active'}
            , 'api_key')
        sub.cancel()
        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/subscriptions/sub_delete/cancel',
            {},
            None
        )


class PlanTest(PayjpResourceTest):

    def test_create_plan(self):
        payjp.Plan.create(**DUMMY_PLAN)

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/plans',
            DUMMY_PLAN,
        )

    def test_delete_plan(self):
        p = payjp.Plan(id="pl_delete")
        p.delete()

        self.requestor_mock.request.assert_called_with(
            'delete',
            '/v1/plans/pl_delete',
            {},
            None
        )

    def test_update_plan(self):
        p = payjp.Plan(id="pl_update")
        p.name = "Plan Name"
        p.save()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/plans/pl_update',
            {
                'name': 'Plan Name',
            },
            None
        )


class RefundTest(PayjpResourceTest):

    def test_non_recursive_save(self):
        charge = payjp.Charge.construct_from({
            'id': 'ch_nested_update',
            'customer': {
                'object': 'customer',
                'description': 'foo',
            },
            'refunds': {
                'object': 'list',
                'url': '/v1/charges/ch_foo/refunds',
                'data': [{
                    'id': 'ref_123',
                }],
            },
        }, 'api_key')

        charge.customer.description = 'bar'
        charge.refunds.has_more = True
        charge.refunds.data[0].description = 'bar'
        charge.save()

        self.assertTrue(self.requestor_mock.request.called)

    def test_fetch_refund(self):
        charge = payjp.Charge.construct_from({
            'id': 'ch_get_refund',
            'refunds': {
                'object': 'list',
                'url': '/v1/charges/ch_get_refund/refunds',
            }
        }, 'api_key')

        charge.refunds.retrieve("ref_get")

        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/charges/ch_get_refund/refunds/ref_get',
            {},
            None
        )

    def test_list_refunds(self):
        charge = payjp.Charge.construct_from({
            'id': 'ch_get_refund',
            'refunds': {
                'object': 'list',
                'url': '/v1/charges/ch_get_refund/refunds',
            }
        }, 'api_key')

        charge.refunds.all()

        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/charges/ch_get_refund/refunds',
            {},
            None
        )


class MetadataTest(PayjpResourceTest):

    def test_noop_metadata(self):
        charge = payjp.Charge(id='ch_foo')
        charge.description = 'test'
        charge.save()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/charges/ch_foo',
            {
                'description': 'test',
            },
            None
        )

    def test_unset_metadata(self):
        charge = payjp.Charge(id='ch_foo')
        charge.metadata = {}
        charge.save()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/charges/ch_foo',
            {
                'metadata': {},
            },
            None
        )

    def test_whole_update(self):
        charge = payjp.Charge(id='ch_foo')
        charge.metadata = {'whole': 'update'}
        charge.save()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/charges/ch_foo',
            {
                'metadata': {'whole': 'update'},
            },
            None
        )

    def test_individual_delete(self):
        charge = payjp.Charge(id='ch_foo')
        charge.metadata = {'whole': None}
        charge.save()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/charges/ch_foo',
            {
                'metadata': {'whole': None},
            },
            None
        )

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

import base64
import calendar
import datetime
import json
import logging
import platform
import time

from six import PY3
from six.moves.urllib.parse import urlencode, urlsplit, urlunsplit

import payjp
from . import (
    error,
    http_client,
    util,
    version,
)

logger = logging.getLogger('payjp')


class APIRequestor(object):

    def __init__(self, key=None, client=None, api_base=None, account=None):
        if api_base:
            self.api_base = api_base
        else:
            self.api_base = payjp.api_base
        self.api_key = key
        self.payjp_account = account

        self._client = client or http_client.new_default_http_client()

    def request(self, method, url, params=None, headers=None):
        body, code, my_api_key = self.request_raw(
            method.lower(), url, params, headers)
        response = self.interpret_response(body, code)
        return response, my_api_key

    def handle_api_error(self, body, code, response):
        try:
            err = response['error']
        except (KeyError, TypeError):
            raise error.APIError(
                "Invalid response object from API: %r (HTTP response code "
                "was %d)" % (body, code),
                body, code, response)

        if code in [400, 404]:
            raise error.InvalidRequestError(
                err.get('message'), err.get('param'), body, code, response)
        elif code == 401:
            raise error.AuthenticationError(
                err.get('message'), body, code, response)
        elif code == 402:
            raise error.CardError(err.get('message'), err.get('param'),
                                  err.get('code'), body, code, response)
        else:
            raise error.APIError(err.get('message'), body, code, response)

    def request_raw(self, method, url, params=None, supplied_headers=None):

        from payjp import api_version

        if self.api_key:
            my_api_key = self.api_key
        else:
            from payjp import api_key
            my_api_key = api_key

        if my_api_key is None:
            raise error.AuthenticationError(
                'No API key provided. (HINT: set your API key using '
                '"payjp.api_key = <API-KEY>"). You can generate API keys '
                'from the Payjp web interface.  See https://docs.pay.jp'
                'for details, or email support@pay.jp if you have any '
                'questions.')

        abs_url = '%s%s' % (self.api_base, url)

        encoded_params = urlencode(list(_api_encode(params or {})))

        if method in ('get', 'delete'):
            if params:
                abs_url = _build_api_url(abs_url, encoded_params)
            post_data = None
        elif method == 'post':
            post_data = encoded_params
        else:
            raise error.APIConnectionError(
                'Unrecognized HTTP method %r.' % (method,))

        ua = {
            'bindings_version': version.VERSION,
            'lang': 'python',
            'publisher': 'payjp',
            'httplib': self._client.name,
        }

        for attr, func in [['lang_version', platform.python_version],
                           ['platform', platform.platform],
                           ['uname', lambda: ' '.join(platform.uname())]]:
            try:
                val = func()
            except Exception as e:
                val = '!! %s' % (e,)
            ua[attr] = val

        if PY3:
            encoded_api_key = str(
                base64.b64encode(
                    bytes(''.join([my_api_key, ':']), 'utf-8')), 'utf-8')
        else:
            encoded_api_key = base64.b64encode(''.join([my_api_key, ':']))

        headers = {
            'X-Payjp-Client-User-Agent': json.dumps(ua),
            'User-Agent': 'Payjp/v1 PythonBindings/%s' % (version.VERSION,),
            'Authorization': 'Basic %s' % encoded_api_key
        }

        if self.payjp_account:
            headers['Payjp-Account'] = self.payjp_account

        if method == 'post':
            headers['Content-Type'] = 'application/x-www-form-urlencoded'

        if api_version is not None:
            headers['Payjp-Version'] = api_version

        if supplied_headers is not None:
            for key, value in supplied_headers.items():
                headers[key] = value

        body, code = self._client.request(
            method, abs_url, headers, post_data)

        logger.info('%s %s %d', method.upper(), abs_url, code)
        logger.debug(
            'API request to %s returned (response code, response body) of '
            '(%d, %r)',
            abs_url, code, body)

        return body, code, my_api_key

    def interpret_response(self, body, code):
        try:
            if hasattr(body, 'decode'):
                body = body.decode('utf-8')
            response = json.loads(body)
        except Exception:
            raise error.APIError(
                "Invalid response body from API: %s "
                "(HTTP response code was %d)" % (body, code),
                body, code)
        if not (200 <= code < 300):
            self.handle_api_error(body, code, response)

        return response

def _encode_datetime(dttime):
    if dttime.tzinfo and dttime.tzinfo.utcoffset(dttime) is not None:
        utc_timestamp = calendar.timegm(dttime.utctimetuple())
    else:
        utc_timestamp = time.mktime(dttime.timetuple())

    return int(utc_timestamp)

def _api_encode(data):
    for key, value in data.items():
        key = util.utf8(key)
        if value is None:
            continue
        elif hasattr(value, 'payjp_id'):
            yield (key, value.payjp_id)
        elif isinstance(value, list) or isinstance(value, tuple):
            for subvalue in value:
                yield ("%s[]" % (key,), util.utf8(subvalue))
        elif isinstance(value, dict):
            subdict = dict(('%s[%s]' % (key, subkey), subvalue) for
                           subkey, subvalue in value.items())
            for subkey, subvalue in _api_encode(subdict):
                yield (subkey, subvalue)
        elif isinstance(value, datetime.datetime):
            yield (key, _encode_datetime(value))
        else:
            yield (key, util.utf8(value))

def _build_api_url(url, query):
    scheme, netloc, path, base_query, fragment = urlsplit(url)

    if base_query:
        query = '%s&%s' % (base_query, query)

    return urlunsplit((scheme, netloc, path, query, fragment))




# coding: utf-8

import unittest
import payjp

from six import string_types

from payjp.test.helper import (PayjpTestCase, NOW, DUMMY_CARD)


class AuthenticationErrorTest(PayjpTestCase):

    def test_invalid_credentials(self):
        key = payjp.api_key
        try:
            payjp.api_key = 'invalid'
            payjp.Customer.create()
        except payjp.error.AuthenticationError as e:
            self.assertEqual(401, e.http_status)
            self.assertTrue(isinstance(e.http_body, string_types))
            self.assertTrue(isinstance(e.json_body, dict))
        finally:
            payjp.api_key = key


class CardErrorTest(PayjpTestCase):

    def test_expired_card_props(self):
        EXPIRED_CARD = DUMMY_CARD.copy()
        EXPIRED_CARD['exp_month'] = NOW.month - 2
        EXPIRED_CARD['exp_year'] = NOW.year - 2
        try:
            payjp.Charge.create(amount=100, currency='jpy', card=EXPIRED_CARD)
        except payjp.error.CardError as e:
            self.assertEqual(402, e.http_status)
            self.assertTrue(isinstance(e.http_body, string_types))
            self.assertTrue(isinstance(e.json_body, dict))


class InvalidRequestErrorTest(PayjpTestCase):

    def test_nonexistent_object(self):
        try:
            payjp.Charge.retrieve('invalid')
        except payjp.error.InvalidRequestError as e:
            self.assertEqual(404, e.http_status)
            self.assertTrue(isinstance(e.http_body, string_types))
            self.assertTrue(isinstance(e.json_body, dict))

    def test_invalid_data(self):
        try:
            payjp.Charge.create()
        except payjp.error.InvalidRequestError as e:
            self.assertEqual(400, e.http_status)
            self.assertTrue(isinstance(e.http_body, string_types))
            self.assertTrue(isinstance(e.json_body, dict))


if __name__ == '__main__':
    unittest.main()



# coding: utf-8


class PayjpException(Exception):
    def __init__(self, message=None, http_body=None, http_status=None,
                 json_body=None):
        super(PayjpException, self).__init__(message)

        if http_body and hasattr(http_body, 'decode'):
            try:
                http_body = http_body.decode('utf-8')
            except:
                http_body = ('<Could not decode body as utf-8. '
                             'Please report to support@pay.jp>')

        self.http_body = http_body

        self.http_status = http_status
        self.json_body = json_body 


class APIError(PayjpException):
    pass


class APIConnectionError(PayjpException):
    pass


class CardError(PayjpException):
    def __init__(self, message, param, code, http_body=None,
                 http_status=None, json_body=None):
        super(CardError, self).__init__(message,
                                        http_body, http_status, json_body)
        self.param = param
        self.code = code 


class AuthenticationError(PayjpException):
    pass


class InvalidRequestError(PayjpException):

    def __init__(self, message, param, http_body=None,
                 http_status=None, json_body=None):
        super(InvalidRequestError, self).__init__(
            message, http_body, http_status, json_body)
        self.param = param



import datetime
import json
import os
import random
import re
import string
import unittest

from mock import patch, Mock

from six import string_types

import payjp

NOW = datetime.datetime.now()

DUMMY_CARD = {
    'number': '4242424242424242',
    'exp_month': NOW.month,
    'exp_year': NOW.year + 4
}

DUMMY_CHARGE = {
    'amount': 100,
    'currency': 'jpy',
    'card': DUMMY_CARD
}

DUMMY_PLAN = {
    'amount': 2000,
    'interval': 'month',
    'name': 'Amazing Gold Plan',
    'currency': 'jpy',
    'id': ('payjp-test-gold-' +
           ''.join(random.choice(string.ascii_lowercase) for x in range(10)))
}

DUMMY_TRANSFER = {
    'amount': 400,
    'currency': 'jpy',
    'recipient': 'self'
}

class PayjpTestCase(unittest.TestCase):
    RESTORE_ATTRIBUTES = ('api_version', 'api_key')

    def setUp(self):
        super(PayjpTestCase, self).setUp()

        self._payjp_original_attributes = {}

        for attr in self.RESTORE_ATTRIBUTES:
            self._payjp_original_attributes[attr] = getattr(payjp, attr)

        api_base = os.environ.get('PAYJP_API_BASE')
        if api_base:
            payjp.api_base = api_base
        payjp.api_key = os.environ.get(
            'PAYJP_API_KEY', 'sk_test_c62fade9d045b54cd76d7036')

    def tearDown(self):
        super(PayjpTestCase, self).tearDown()

        for attr in self.RESTORE_ATTRIBUTES:
            setattr(payjp, attr, self._payjp_original_attributes[attr])

    # Python < 2.7 compatibility
    def assertRaisesRegexp(self, exception, regexp, callable, *args, **kwargs):
        try:
            callable(*args, **kwargs)
        except exception as err:
            if regexp is None:
                return True

            if isinstance(regexp, string_types):
                regexp = re.compile(regexp)
            if not regexp.search(str(err)):
                raise self.failureException('"%s" does not match "%s"' %
                                            (regexp.pattern, str(err)))
        else:
            raise self.failureException(
                '%s was not raised' % (exception.__name__,))


class PayjpUnitTestCase(PayjpTestCase):
    REQUEST_LIBRARIES = ['requests']

    def setUp(self):
        super(PayjpUnitTestCase, self).setUp()

        self.request_patchers = {}
        self.request_mocks = {}
        for lib in self.REQUEST_LIBRARIES:
            patcher = patch("payjp.http_client.%s" % (lib,))

            self.request_mocks[lib] = patcher.start()
            self.request_patchers[lib] = patcher

    def tearDown(self):
        super(PayjpUnitTestCase, self).tearDown()

        for patcher in self.request_patchers.values():
            patcher.stop()


class PayjpApiTestCase(PayjpTestCase):

    def setUp(self):
        super(PayjpApiTestCase, self).setUp()

        self.requestor_patcher = patch('payjp.api_requestor.APIRequestor')
        self.requestor_class_mock = self.requestor_patcher.start()
        self.requestor_mock = self.requestor_class_mock.return_value

    def tearDown(self):
        super(PayjpApiTestCase, self).tearDown()

        self.requestor_patcher.stop()

    def mock_response(self, res):
        self.requestor_mock.request = Mock(return_value=(res, 'reskey'))


class MyResource(payjp.resource.APIResource):
    pass


class MyListable(payjp.resource.ListableAPIResource):
    pass


class MyCreatable(payjp.resource.CreateableAPIResource):
    pass


class MyUpdateable(payjp.resource.UpdateableAPIResource):
    pass


class MyDeletable(payjp.resource.DeletableAPIResource):
    pass


class MyComposite(payjp.resource.ListableAPIResource,
                  payjp.resource.CreateableAPIResource,
                  payjp.resource.UpdateableAPIResource,
                  payjp.resource.DeletableAPIResource):
    pass



# coding: utf-8

import sys

from setuptools import setup

install_requires = []

if sys.version_info < (2, 7):
    raise DeprecationWarning('Python 2.6 and older are no longer supported by PAY.JP. ')

install_requires.append('requests >= 2.7.0')
install_requires.append('six >= 1.9.0')

setup(
    name="payjp",
    version="0.0.4",
    description='PAY.JP python bindings',
    author="PAY.JP",
    author_email='support@pay.jp',
    packages=['payjp', 'payjp.test'],
    url='https://github.com/payjp/payjp-python',
    install_requires=install_requires,
    tests_require=[
        'mock >= 1.3.0'
        ],
    test_suite='payjp.test.all',
)



import pkgutil
import unittest


def all_names():
    for _, modname, _ in pkgutil.iter_modules(__path__):
        if modname.startswith('test_'):
            yield 'payjp.test.' + modname


def all():
    return unittest.defaultTestLoader.loadTestsFromNames(all_names())


def unit():
    unit_names = [name for name in all_names() if 'integration' not in name]
    return unittest.defaultTestLoader.loadTestsFromNames(unit_names)



# coding: utf-8

import unittest

from mock import Mock

import payjp

from payjp.test.helper import PayjpUnitTestCase

VALID_API_METHODS = ('get', 'post', 'delete')


class HttpClientTests(PayjpUnitTestCase):

    def setUp(self):
        super(HttpClientTests, self).setUp()

        self.original_filters = payjp.http_client.warnings.filters[:]
        payjp.http_client.warnings.simplefilter('ignore')

    def tearDown(self):
        payjp.http_client.warnings.filters = self.original_filters

        super(HttpClientTests, self).tearDown()

    def check_default(self, none_libs, expected):
        for lib in none_libs:
            setattr(payjp.http_client, lib, None)

        inst = payjp.http_client.new_default_http_client()

        self.assertTrue(isinstance(inst, expected))

    def test_new_default_http_client_requests(self):
        self.check_default((),
                           payjp.http_client.RequestsClient)


class ClientTestBase():

    @property
    def request_mock(self):
        return self.request_mocks[self.request_client.name]

    @property
    def valid_url(self, path='/foo'):
        return 'https://api.pay.jp%s' % (path,)

    def make_request(self, method, url, headers, post_data):
        client = self.request_client()
        return client.request(method, url, headers, post_data)

    def mock_response(self, body, code):
        raise NotImplementedError(
            'You must implement this in your test subclass')

    def mock_error(self, error):
        raise NotImplementedError(
            'You must implement this in your test subclass')

    def check_call(self, meth, abs_url, headers, params):
        raise NotImplementedError(
            'You must implement this in your test subclass')

    def test_request(self):
        self.mock_response(self.request_mock, '{"foo": "baz"}', 200)

        for meth in VALID_API_METHODS:
            abs_url = self.valid_url
            data = ''

            if meth != 'post':
                abs_url = '%s?%s' % (abs_url, data)
                data = None

            headers = {'my-header': 'header val'}

            body, code = self.make_request(
                meth, abs_url, headers, data)

            self.assertEqual(200, code)
            self.assertEqual('{"foo": "baz"}', body)

            self.check_call(self.request_mock, meth, abs_url,
                            data, headers)

    def test_exception(self):
        self.mock_error(self.request_mock)
        self.assertRaises(payjp.error.APIConnectionError,
                          self.make_request,
                          'get', self.valid_url, {}, None)


class RequestsClientTests(PayjpUnitTestCase, ClientTestBase):
    request_client = payjp.http_client.RequestsClient

    def mock_response(self, mock, body, code):
        result = Mock()
        result.content = body
        result.status_code = code

        mock.request = Mock(return_value=result)

    def mock_error(self, mock):
        mock.exceptions.RequestException = Exception
        mock.request.side_effect = mock.exceptions.RequestException()

    def check_call(self, mock, meth, url, post_data, headers):
        mock.request.assert_called_with(meth, url,
                                        headers=headers,
                                        data=post_data,
                                        timeout=80)


if __name__ == '__main__':
    unittest.main()
